from pcappy import PcapPyOffline, open_offline
import struct
import subprocess
import os
import re
import sys
import glob

from read_airodump import csv2blob
from parse_aps import parse_aps

DRYRUN=False

"""
prep_some_lines.py

Prepare some handshake capture files 
to be run through a password cracker.

This involves finding cap files,
finding cap files with handshakes,
finding info about whose handshakes,
converting handshakes to proper format,
and lining everything up in a list
for easy cracking.

Note that these commands may be run 
on a separate machine, so the processing
and output should ideally bundle things up
together.

(e.g., a list of John password files should
also come with the actual password files.)
"""

def usage():
    a  = "\n" 
    a += "\nprep_some_lines.py" 
    a += "\n" 
    a += "\nLine up some captured handshake packets for cracking."
    a += "\n" 
    a += "\nFeed it the directories containing pcap files."
    a += "\n"
    a += "\nUsage:"
    a += "\n    python prep_some_lines.py [dir1] [dir2] [...]"
    a += "\n"
    a += "\ne.g., python prep_some_lines.py /root/dumps/2015-08-15/"
    a += "\n"
    raise Exception(a)

AIRCRACK_BIN    = '/usr/bin/aircrack-ng'
WPACLEAN_BIN    = '/usr/bin/wpaclean'

KRONOS_JOHN_BIN_DIR = "/root/codes/_sources/JohnTheRipper-1.8.0.6-jumbo-1/run/"
#ROJO_JOHN_BIN_DIR    = "/home/charles/codes/_sources/john-1.7.9-jumbo-7/run/"
JOHN_BIN_DIR    = KRONOS_JOHN_BIN_DIR
H2JBIN          = JOHN_BIN_DIR + "hccap2john"





# -----------------------------------------------
# -----------------------------------------------
# Now we start
# -----------------------------------------------
# -----------------------------------------------
    
    
def main():

    try:
        args = sys.argv
        assert len(args)>1
    except AssertionError:
        usage()
    
    basedirs = sys.argv[1:]


    hccap_files = extract_cap_handshakes(basedirs)
    
    
    export_handshakes_john(hccap_files)



# -----------------------------------------------
# -----------------------------------------------
# Now we define how to extract handshakes from caps
# -----------------------------------------------
# -----------------------------------------------

def extract_cap_handshakes(basedirs):
    """
    Look for cap files with handshakes,
    and extract them to hccap files.
    Does not return anything.
    """


    ###########################################
    # Look for cap/csv handshakes
    
    
    print ""
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print ""
    print "Beginning search for cap files with handshakes"
    print ""
    
    
    handshakes_found = 0
    
    hccap_files = []
    
    for basedir in basedirs:
    
        if basedir[-1] <> '/':
            basedir += '/'
    
        cap_files=[]
        csv_files=[]
        
        for f in os.listdir(basedir):
            name,ext = os.path.splitext(f)
            if ext==".cap":
                if os.path.isfile(basedir+name+".csv"):
                    cap_files.append(basedir+name+".cap")
                    csv_files.append(basedir+name+".csv")
                else: 
                    cap_files.append(basedir+name+".cap")
    
        print ""
        print "[ ] Looking for cap files with handshakes in",basedir
    
    
        for iic, cap_file in enumerate(cap_files):


            f = open(os.devnull, 'w')
            s = subprocess.Popen([WPACLEAN_BIN, '/tmp/cleaned.cap'] + [cap_file], stdout=f, stderr=f)
            s.wait()
            if os.stat('/tmp/cleaned.cap').st_size == 24:
                print '[-] No WPA/WPA2 handshakes captured from %s'%cap_file
                pass
            
            p = open_offline('/tmp/cleaned.cap')
            # filter beacons
            p.filter = 'link[0] == 0x80'
            
            def gotpacket(d, hdr, data):
                bssid = struct.unpack('6B', data[10:16])
                bssid_str = ':'.join(format(x, '02x') for x in bssid)
    
                d.append(bssid_str)
            
            # Parameters are count, callback, user params
            bssid_list = []
            p.loop(-1, gotpacket, bssid_list)


            ##############################33
            # Our BSSID list from the cap file is a 
            # short but interesting set of BSSIDs
            # whose handshakes have been captured.
            #
            for bssid in bssid_list:
        
                BSSID = bssid.upper()
    
                # this is a new bssid
                print '[+] Handshake found!'
                print '    pcap file = %s'%cap_file
                print '    BSSID = %s'%BSSID


                bssid_file = re.sub(r'\:','_',BSSID)
    

                # check if we have a csv file, 
                # airodump-ng dumps a csv file
                # besside-ng does not 
                if csv_files <> []:

                    csv_file = csv_files[iic]
                
                    # extract bssid/essid information
                    stations_list, clients_list = csv2blob(csv_file)
    
                    # stations
                    stations_head, stations_data = parse_aps(stations_list)
    
                    bssid_ix = stations_head.index("BSSID")
                    essid_ix = stations_head.index("ESSID")

                    essid = ''
    
                    all_bssids = [station[bssid_ix].strip() for station in stations_data]
                    all_essids = [station[essid_ix].strip() for station in stations_data]
    
                    if BSSID in all_bssids:
                        this_ix = all_bssids.index(BSSID)
                        this_essid = all_essids[this_ix].strip()
    
                    try:
                        print '    ESSID = %s'%essid
                    except:
                        print '    ESSID N/A'

            
    
                print "    Extracting BSSID handshakes to hccap file [%s] with Aircrack"%( '/tmp'+bssid_file )
                if not DRYRUN:
                    s = subprocess.call([AIRCRACK_BIN, '-J', '/tmp/'+bssid_file, '-b', BSSID, '/tmp/cleaned.cap'],stdout=f,stdin=f)

                src = "/tmp/"+bssid_file+".hccap"
                dest = basedir+bssid_file+".hccap"

                print "    Moving hccap handshake file to [%s]"%(dest)
                if not DRYRUN:
                    s2 = subprocess.call(["cp","-f",src,dest],stdout=f,stdin=f)

                hccap_files.append(dest)
   
                handshakes_found += 1


    
    print ""
    print "Found %d handshakes."%handshakes_found
    
    print ""
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print ""

    return hccap_files






# -----------------------------------------------
# -----------------------------------------------
# Now we define how to convert hccaps to john files
# -----------------------------------------------
# -----------------------------------------------

def export_handshakes_john(hccap_files):
    """
    Export handshakes from hccap file 
    into John password file format. 

    This takes a list of hccap files 
    (generated from the extraction process)
    and performs the conversion of 
    hccap to john files.
    
    The John password file is put in the same 
    place as the hccap file, which is put in 
    the same place as the original cap file.
    """


    ###########################################
    # Look for cap/csv handshakes
    
    print ""
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print ""
    print "Convert hccap files to John:"


    john_files = []
    
    for hccap_file in hccap_files:
    
        john_pw_file = re.sub(r'hccap','johnpw',hccap_file)

        
        # This is a new hccap file.
        print '[+] hccap file being processed.'
        print '    hccap file = %s'%(hccap_file)


        # hccap 2 john command:
        h2j_cmd = [H2JBIN,hccap_file]#,'|','tee',john_pw_file]
        
        print '    %s'%( ' '.join(h2j_cmd) )


        # actually run the 
        # hccap 2 john command:

        print '    Running hccap2john to convert to John password file'
        
        if not DRYRUN:
            with open(john_pw_file, 'w') as f:
                s = subprocess.call(h2j_cmd, stdout=f)

        john_files.append(john_pw_file)
        print '    John the Ripper pw file = %s'%(john_pw_file)
        print '\n'
    
    print ""
    print "Okay, all done."
    print ""
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print "*** "*30
    print ""

    return john_files


if __name__=="__main__":
    main()

